'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = createRouter;

function createRouter(history, transition) {
  var listeners = [];
  var queue = [];
  var waiting = [];
  var inProcess = false;

  function listen(listener) {
    listeners.push(listener);

    return function unlisten() {
      listeners = listeners.filter(function (l) {
        return l !== listener;
      });
    };
  }

  function navigateTo(location) {
    return transition(location).then(function (screen) {
      listeners.forEach(function (listener) {
        if (screen) {
          listener([location, screen]);
        }
      });
    });
  }

  function processQueue() {
    if (inProcess) {
      return;
    }

    if (queue.length === 0) {
      waiting.forEach(function (resolve) {
        return resolve();
      });
      waiting = [];
      return;
    }

    var location = queue.shift();
    inProcess = true;
    navigateTo(location).then(function () {
      inProcess = false;
      processQueue();
    })['catch'](function (error) {
      console.error(error.stack);
      inProcess = false;
      processQueue();
    });
  }

  function waitQueue() {
    return new Promise(function (resolve) {
      waiting.push(resolve);
    });
  }

  history.listen(function (location) {
    // Reset the queue to skip unreachable transitions
    queue = [location];
    processQueue();
  });

  return {
    listen: listen,
    waitQueue: waitQueue
  };
}

module.exports = exports['default'];