'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = createRouterEngine;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _composeMiddlewares = require('./composeMiddlewares');

var _composeMiddlewares2 = _interopRequireDefault(_composeMiddlewares);

function createRouterEngine(router, history) {
  var middlewares = arguments.length <= 2 || arguments[2] === undefined ? [] : arguments[2];

  var queue = [];
  var waiting = [];
  var inProcess = false;

  function execute(location) {
    return router.navigateTo(location).then(function () {
      return location;
    });
  }

  var dispatch = (0, _composeMiddlewares2['default'])(middlewares, execute);

  function processQueue() {
    if (inProcess) {
      return;
    }

    if (queue.length === 0) {
      waiting.forEach(function (resolve) {
        return resolve();
      });
      waiting = [];
      return;
    }

    var location = queue.shift();
    inProcess = true;
    dispatch(location).then(function () {
      inProcess = false;
      processQueue();
    })['catch'](function (error) {
      console.error(error.stack);
      inProcess = false;
      processQueue();
    });
  }

  function waitQueue() {
    return new Promise(function (resolve) {
      waiting.push(resolve);
    });
  }

  function run() {
    history.listen(function (location) {
      // Reset the queue to skip unreachable transitions
      queue = [location];
      processQueue();
    });
  }

  return {
    waitQueue: waitQueue,
    run: run
  };
}

module.exports = exports['default'];