/* @qflow */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _slicedToArray = (function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i['return']) _i['return'](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError('Invalid attempt to destructure non-iterable instance'); } }; })();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.routerReducer = routerReducer;
exports.historyMiddleware = historyMiddleware;
exports.pushState = pushState;
exports.replaceState = replaceState;
exports.transitionTo = transitionTo;
exports.createRouterListener = createRouterListener;
var PUSH_STATE = '@@vstack-router/PUSH_STATE';
var REPLACE_STATE = '@@vstack-router/REPLACE_STATE';
var TRANSITION_TO = '@@vstack-router/TRANSITION_TO';
var SCREEN_CHANGE = '@@vstack-router/SCREEN_CHANGE';

var initialState = {
  location: undefined,
  screen: undefined
};

function routerReducer(state, action) {
  if (state === undefined) state = initialState;

  if (action.type === SCREEN_CHANGE) {
    return _extends({}, state, {
      location: action.location,
      screen: action.screen
    });
  } else {
    return state;
  }
}

function historyMiddleware(history) {
  return function (store) {
    return function (next) {
      return function (action) {
        switch (action.type) {
          case PUSH_STATE:
            history.pushState(action.state, action.url);
            break;

          case REPLACE_STATE:
            history.replaceState(action.state, action.url);
            break;

          case TRANSITION_TO:
            history.transitionTo(action.location);
            break;

          default:
            return next(action);
        }
      };
    };
  };
}

function pushState(state, url) {
  return {
    type: PUSH_STATE,
    state: state,
    url: url
  };
}

function replaceState(state, url) {
  return {
    type: REPLACE_STATE,
    state: state,
    url: url
  };
}

function transitionTo(location) {
  return {
    type: TRANSITION_TO,
    location: location
  };
}

function createRouterListener(store) {
  return function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2);

    var location = _ref2[0];
    var screen = _ref2[1];
    return store.dispatch({
      type: SCREEN_CHANGE,
      location: location,
      screen: screen
    });
  };
}